﻿Imports FarPoint.Win.Spread
Imports FarPoint.Win.Spread.CellType
Imports System
Imports System.Collections.Generic
Imports System.Drawing
Imports System.Text

Public Class AnimateCellType
    Inherits ImageCellType
    'Implements IDisposable
#Region "Variables and Initializing"
    Private currentImage As Image = Nothing
    Private sheet As SheetView = Nothing
    Private rowIndex As Integer = -1
    Private columnIndex As Integer = -1

    ''' <summary>
    ''' Constructor
    ''' </summary>
    ''' <param name="owner">Need to notify invalidate, check and control Spread from celltype</param>
    ''' <param name="row">row index</param>
    ''' <param name="column">column index</param>
    Public Sub New(owner As SheetView, row As Integer, column As Integer)
        If owner Is Nothing Then
            Throw New ArgumentNullException("owner")
        End If
        If owner.FpSpread Is Nothing Then
            Throw New ArgumentNullException("owner.FpSpread")
        End If
        If row < 0 OrElse row >= owner.RowCount Then
            Throw New ArgumentOutOfRangeException("row")
        End If
        If column < 0 OrElse column >= owner.ColumnCount Then
            Throw New ArgumentOutOfRangeException("column")
        End If
        sheet = owner
        rowIndex = row
        columnIndex = column
        AddHandler sheet.Disposing, AddressOf sheet_Disposing
        AddHandler sheet.CellChanged, AddressOf sheet_CellChanged
        AddHandler sheet.FpSpread.ActiveSheetChanged, AddressOf FpSpread_ActiveSheetChanged
        AddHandler sheet.Disposed, AddressOf FpSpread_Disposed
    End Sub
#End Region
#Region "Dispose"
    Private Sub FpSpread_Disposed(sender As Object, e As EventArgs)
        StopAnimate()
        RemoveHandler sheet.FpSpread.Disposed, AddressOf FpSpread_Disposed
        RemoveHandler sheet.FpSpread.ActiveSheetChanged, AddressOf FpSpread_ActiveSheetChanged
    End Sub
    Private Sub sheet_Disposing(sender As Object, e As EventArgs)
        StopAnimate()
        RemoveHandler sheet.Disposing, AddressOf sheet_Disposing
        RemoveHandler sheet.CellChanged, AddressOf sheet_CellChanged
    End Sub
    Public Sub Dispose()
        StopAnimate()
        RemoveHandler sheet.FpSpread.Disposed, AddressOf FpSpread_Disposed
        RemoveHandler sheet.FpSpread.ActiveSheetChanged, AddressOf FpSpread_ActiveSheetChanged
        RemoveHandler sheet.Disposing, AddressOf sheet_Disposing
        RemoveHandler sheet.CellChanged, AddressOf sheet_CellChanged
    End Sub
#End Region
    ''' <summary>
    ''' Should not animate when cell is not display
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub FpSpread_ActiveSheetChanged(sender As Object, e As EventArgs)
        If Not sheet.FpSpread.ActiveSheet Is sheet Then
            StopAnimate()
        End If
    End Sub
    ''' <summary>
    ''' Should update in case value changed
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub sheet_CellChanged(sender As Object, e As SheetViewEventArgs)
        If e.Row = rowIndex AndAlso e.Column = columnIndex Then
            StopAnimate()
        End If
    End Sub
    ''' <summary>
    ''' Stop Animate
    ''' </summary>
    Private Sub StopAnimate()
        If Not currentImage Is Nothing Then
            SyncLock currentImage
                ImageAnimator.StopAnimate(currentImage, AddressOf OnFrameChanged)
                currentImage.Dispose()
                currentImage = Nothing
            End SyncLock
        End If
    End Sub
    ''' <summary>
    ''' Start Animate
    ''' </summary>
    Private Sub StartAnimate(value As Object)
        StopAnimate()
        Dim imgValue As Image = ImageCellType.GetImage(value)
        If imgValue IsNot Nothing AndAlso ImageAnimator.CanAnimate(imgValue) Then
            currentImage = TryCast(imgValue.Clone(), Image)
            ImageAnimator.Animate(currentImage, AddressOf OnFrameChanged)
        End If
    End Sub
    ''' <summary>
    ''' Original cell Paint or repaint cause by image frame per second rate
    ''' </summary>
    ''' <param name="g"></param>
    ''' <param name="r"></param>
    ''' <param name="appearance"></param>
    ''' <param name="value"></param>
    ''' <param name="isSelected"></param>
    ''' <param name="isLocked"></param>
    ''' <param name="zoomFactor"></param>
    Public Overrides Sub PaintCell(g As Graphics, r As Rectangle, appearance As FarPoint.Win.Spread.Appearance, value As Object, isSelected As Boolean, isLocked As Boolean, _
     zoomFactor As Single)
        If currentImage Is Nothing Then
            MyBase.PaintCell(g, r, appearance, value, isSelected, isLocked, zoomFactor)
            StartAnimate(value)
        Else
            SyncLock currentImage
                MyBase.PaintCell(g, r, appearance, currentImage, isSelected, isLocked, zoomFactor)
            End SyncLock
        End If
    End Sub
    ''' <summary>
    ''' depend on frame per second rate, it will notify to update frame for painting
    ''' </summary>
    ''' <param name="sender"></param>
    ''' <param name="e"></param>
    Private Sub OnFrameChanged(sender As Object, e As EventArgs)
        SyncLock currentImage
            ImageAnimator.UpdateFrames(currentImage)
        End SyncLock
        Dim rv As Integer = sheet.FpSpread.GetRowViewportCount()
        Dim cv As Integer = sheet.FpSpread.GetColumnViewportCount()
        Dim spreadRect As Rectangle = sheet.FpSpread.Bounds
        For r As Integer = -1 To rv - 1
            For c As Integer = -1 To cv - 1
                Dim rect As Rectangle = sheet.FpSpread.GetCellRectangle(r, c, rowIndex, columnIndex)
                rect.Intersect(spreadRect)
                If Not rect.IsEmpty Then
                    sheet.FpSpread.Invalidate()
                End If
            Next
        Next
    End Sub

End Class
